"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildExpression = buildExpression;
exports.buildRuntime = buildRuntime;

var _core = _interopRequireDefault(require("../core"));

var tools = _interopRequireWildcard(require("../utils/tools"));

var tranData = _interopRequireWildcard(require("../transforms/transformData"));

var _escape = require("../utils/escape");

var _extension = require("../helpers/extension");

var _filter = require("../helpers/filter");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var GLOBAL = 'g';
var CONTEXT = 'c';
var PARAMS = 'p';

function _buildFn(content, node, fns, no, newContext, level, useStringLocal) {
  var fnStr = '';
  var useString = useStringLocal != null ? useStringLocal : fns.useString,
      main = no === 0,

  /* retType
   1: single child node
   2: multiple child nodes
   object: not build function
  */
  retType = content.length === 1 ? '1' : '2',
      counter = {
    _type: 0,
    _params: 0,
    _compParam: 0,
    _children: 0,
    _dataRefer: 0,
    _ex: 0,
    _value: 0,
    _filter: 0,
    _fnH: 0,
    _tmp: 0,
    newContext: newContext
  };

  if (!useString) {
    counter._compParam = 0;
  } else {
    counter._children = 0;
  }

  if (!main && newContext) {
    fnStr += "".concat(CONTEXT, " = ").concat(GLOBAL, ".n(").concat(CONTEXT, ", ").concat(PARAMS, ");\n");
  }

  if (retType === '2') {
    if (!useString) {
      fnStr += 'var ret = [];\n';
    } else {
      fnStr += "var ret = '';\n";
    }
  }

  fnStr += _buildContent(content, node, fns, counter, retType, level, useStringLocal);

  if (retType === '2') {
    fnStr += 'return ret;';
  }

  try {
    /* build template functions
     g: global configs
     c: context
     p: parameters
    */
    fns[main ? 'main' : 'fn' + no] = new Function(GLOBAL, CONTEXT, PARAMS, fnStr);
  } catch (err) {
    tools.error('Failed to generate template function:\n\n' + err.toString() + ' in\n\n' + fnStr + '\n');
  }

  return no;
}

function _buildOptions(config, useStringLocal, node, fns, level, hashProps, tagName, tagProps) {
  var hashStr = ', useString: ' + (useStringLocal == null ? "".concat(GLOBAL, ".us") : useStringLocal ? 'true' : 'false');
  var noConfig = !config,
      no = fns._no;

  if (node) {
    //tags
    var newContext = config ? config.newContext : true;
    var isDirective = node.isDirective || config && config.isDirective;

    if (noConfig || config.hasName) {
      hashStr += ", name: '".concat(node.ex, "'");
    }

    if (tagName && isDirective && (noConfig || !config.noTagName)) {
      hashStr += ', tagName: ' + tagName;
      hashStr += ", setTagName: function(c) { ".concat(tagName, " = c }");
    }

    if (tagProps && (noConfig || config.hasTagProps)) {
      hashStr += ', tagProps: ' + tagProps;
    }

    if (hashProps != null) {
      hashStr += ', props: ' + hashProps;
    }

    hashStr += ', ' + (isDirective ? 'value' : 'children') + ': ' + (node.content ? "".concat(GLOBAL, ".r(").concat(GLOBAL, ", ").concat(CONTEXT, ", ").concat(GLOBAL, ".fn") + _buildFn(node.content, node, fns, ++fns._no, newContext, level, useStringLocal) + ')' : "".concat(GLOBAL, ".np"));
  }

  return '{ _njOpts: ' + (no + 1) + (noConfig || config.hasTmplCtx ? ", global: ".concat(GLOBAL, ", context: ").concat(CONTEXT) : '') + (noConfig || config.hasOutputH ? ', outputH: ' + !fns.useString : '') + hashStr + (level != null && (noConfig || config.hasLevel) ? ', level: ' + level : '') + ' }';
}

var CUSTOM_VAR = 'nj_custom';
var OPERATORS = ['+', '-', '*', '/', '%', '===', '!==', '==', '!=', '<=', '>=', '=', '+=', '<', '>', '&&', '||', '?', ':'];
var ASSIGN_OPERATORS = ['=', '+='];
var SP_FILTER_REPLACE = {
  or: '||'
};

function _buildDataValue(ast, escape, fns, level) {
  var dataValueStr,
      special = false;
  var isBasicType = ast.isBasicType,
      isAccessor = ast.isAccessor,
      hasSet = ast.hasSet;

  if (isBasicType) {
    dataValueStr = ast.name;
  } else {
    var name = ast.name,
        parentNum = ast.parentNum;
    var data = '',
        specialP = false;

    switch (name) {
      case '@index':
        data = 'index';
        special = true;
        break;

      case '@item':
        data = 'item';
        special = true;
        break;

      case 'this':
        data = 'data';

        special = function special(data) {
          return "".concat(data, "[").concat(data, ".length - 1]");
        };

        break;

      case '@data':
        data = 'data';
        special = true;
        break;

      case '@g':
        data = "".concat(GLOBAL, ".g");
        special = CUSTOM_VAR;
        break;

      case '@context':
        data = CONTEXT;
        special = CUSTOM_VAR;
        break;

      case '@lt':
        data = "'<'";
        special = CUSTOM_VAR;
        break;

      case '@gt':
        data = "'>'";
        special = CUSTOM_VAR;
        break;

      case '@lb':
        data = "'{'";
        special = CUSTOM_VAR;
        break;

      case '@rb':
        data = "'}'";
        special = CUSTOM_VAR;
        break;

      case '@q':
        data = "'\"'";
        special = CUSTOM_VAR;
        break;

      case '@sq':
        data = "\"'\"";
        special = CUSTOM_VAR;
        break;
    }

    if (parentNum) {
      if (!data) {
        data = 'data';
      }

      var isCtx = data == CONTEXT;

      for (var i = 0; i < parentNum; i++) {
        data = !isCtx ? 'parent.' + data : data + '.parent';
      }

      if (!special) {
        specialP = true;
      }
    }

    if (!special && !specialP) {
      dataValueStr = (isAccessor ? "".concat(GLOBAL, ".c(") : '') + "".concat(CONTEXT, ".d('") + name + "'" + (hasSet ? ', 0, true' : '') + ')' + (isAccessor ? ", ".concat(CONTEXT) + ')' : '');
    } else {
      var isCustomVar = special === CUSTOM_VAR;
      var dataStr = isCustomVar ? data : "".concat(CONTEXT, ".") + data;

      if (tools.isObject(special)) {
        dataStr = special(dataStr);
      }

      dataValueStr = special ? isCustomVar || !hasSet ? dataStr : "{ source: c, value: ".concat(dataStr, ", prop: '").concat(data, "', _njSrc: true }") : (isAccessor ? "".concat(GLOBAL, ".c(") : '') + "".concat(CONTEXT, ".d('") + name + "', " + dataStr + (hasSet ? ', true' : '') + ')' + (isAccessor ? ", ".concat(CONTEXT) + ')' : '');
    }
  }

  if (dataValueStr) {
    dataValueStr = _replaceBackslash(dataValueStr);
  }

  return _buildEscape(dataValueStr, fns, isBasicType || isAccessor ? false : escape, special);
}

function replaceFilterName(name) {
  var nameR = SP_FILTER_REPLACE[name];
  return nameR != null ? nameR : name;
}

function buildExpression(ast, inObj, escape, fns, useStringLocal, level) {
  var codeStr = ast.filters && OPERATORS.indexOf(replaceFilterName(ast.filters[0].name)) < 0 ? '' : !inObj ? _buildDataValue(ast, escape, fns, level) : ast.name;
  var lastCodeStr = '';
  ast.filters && ast.filters.forEach(function (filter, i) {
    var hasFilterNext = ast.filters[i + 1] && OPERATORS.indexOf(replaceFilterName(ast.filters[i + 1].name)) < 0;
    var filterName = replaceFilterName(filter.name);

    if (OPERATORS.indexOf(filterName) >= 0) {
      //Native operator
      if (ASSIGN_OPERATORS.indexOf(filterName) >= 0) {
        codeStr += ".source.".concat(i == 0 ? ast.name : tools.clearQuot(ast.filters[i - 1].params[0].name), " ").concat(filterName, " ");
      } else {
        codeStr += " ".concat(filterName, " ");
      }

      if (!ast.filters[i + 1] || OPERATORS.indexOf(replaceFilterName(ast.filters[i + 1].name)) >= 0) {
        if (filter.params[0].filters) {
          codeStr += '(';
          codeStr += buildExpression(filter.params[0], null, escape, fns, useStringLocal, level);
          codeStr += ')';
        } else {
          codeStr += _buildDataValue(filter.params[0], escape, fns, level);
        }
      }
    } else if (filterName === '_') {
      //Call function
      var _codeStr = "".concat(GLOBAL, ".f['").concat(filterName, "'](").concat(lastCodeStr);

      if (filter.params.length) {
        _codeStr += ', [';
        filter.params.forEach(function (param, j) {
          _codeStr += buildExpression(param, null, escape, fns, useStringLocal, level);

          if (j < filter.params.length - 1) {
            _codeStr += ', ';
          }
        });
        _codeStr += ']';
      }

      _codeStr += ')';

      if (hasFilterNext) {
        lastCodeStr = _codeStr;
      } else {
        codeStr += _codeStr;
        lastCodeStr = '';
      }
    } else {
      //Custom filter
      var startStr, endStr, isObj, configF;
      var isMethod = ast.isEmpty && i == 0;

      if (filterName === 'bracket') {
        startStr = '(';
        endStr = ')';
      } else if (filterName === 'list') {
        startStr = '[';
        endStr = ']';
      } else if (filterName === 'obj') {
        startStr = '{ ';
        endStr = ' }';
        isObj = true;
      } else {
        if (filterName == 'require') {
          startStr = 'require';
        } else {
          var filterStr = "".concat(GLOBAL, ".f['").concat(filterName, "']"),
              warnStr = "".concat(GLOBAL, ".wn('").concat(filterName, "', 'f')"),
              isDev = process.env.NODE_ENV !== 'production';
          configF = _filter.filterConfig[filterName];

          if (configF && configF.onlyGlobal) {
            startStr = isDev ? "(".concat(filterStr, " || ").concat(warnStr, ")") : filterStr;
          } else {
            startStr = "".concat(GLOBAL, ".cf(").concat(CONTEXT, ".d('").concat(filterName, "', 0, true) || ").concat(filterStr).concat(isDev ? " || ".concat(warnStr) : '', ")");
          }
        }

        startStr += '(';
        endStr = ')';
      }

      var _codeStr2 = startStr;

      if (isMethod) {
        //Method
        filter.params.forEach(function (param, j) {
          _codeStr2 += buildExpression(param, isObj, escape, fns, useStringLocal, level);

          if (j < filter.params.length - 1) {
            _codeStr2 += ', ';
          }
        });
      } else {
        //Operator
        if (i == 0) {
          _codeStr2 += _buildDataValue(ast, escape, fns, level);
        } else if (lastCodeStr !== '') {
          _codeStr2 += lastCodeStr;
        } else {
          if (ast.filters[i - 1].params[0].filters) {
            _codeStr2 += buildExpression(ast.filters[i - 1].params[0], null, escape, fns, useStringLocal, level);
          } else {
            _codeStr2 += _buildDataValue(ast.filters[i - 1].params[0], escape, fns, level);
          }
        }

        filter.params && filter.params.forEach(function (param, j) {
          _codeStr2 += ', ';

          if (param.filters) {
            _codeStr2 += buildExpression(param, null, escape, fns, useStringLocal, level);
          } else {
            _codeStr2 += _buildDataValue(param, escape, fns, level);
          }
        });
        var nextFilter = ast.filters[i + 1];

        if (filterName === '.' && nextFilter && replaceFilterName(nextFilter.name) === '_') {
          _codeStr2 += ', true';
        } //if (!configF || configF.hasOptions) {


        if (configF && configF.hasOptions) {
          _codeStr2 += ", ".concat(_buildOptions(configF, useStringLocal, null, fns, level));
        }
      }

      _codeStr2 += endStr;

      if (hasFilterNext) {
        lastCodeStr = _codeStr2;
      } else {
        codeStr += _codeStr2;
        lastCodeStr = '';
      }
    }
  });
  return codeStr;
}

function _buildEscape(valueStr, fns, escape, special) {
  if (fns.useString) {
    if (escape && special !== CUSTOM_VAR) {
      return "".concat(GLOBAL, ".es(") + valueStr + ')';
    } else {
      return valueStr;
    }
  } else {
    //文本中的特殊字符需转义
    return (0, _escape.unescape)(valueStr);
  }
}

function _replaceStrs(str) {
  return _replaceBackslash(str).replace(/_njNl_/g, '\\n').replace(/'/g, "\\'");
}

function _replaceBackslash(str) {
  return str = str.replace(/\\/g, '\\\\');
}

function _buildProps(obj, fns, useStringLocal, level) {
  var str0 = obj.strs[0];
  var valueStr = '';

  if (tools.isString(str0)) {
    //常规属性
    valueStr = !obj.isAll && str0 !== '' ? "'".concat(_replaceStrs(str0), "'") : '';
    tools.each(obj.props, function (o, i) {
      var dataValueStr = buildExpression(o.prop, null, o.escape, fns, useStringLocal, level);

      if (!obj.isAll) {
        var strI = obj.strs[i + 1],
            prefixStr = str0 === '' && i == 0 ? '' : ' + '; // if (strI.trim() === '\\n') { //如果只包含换行符号则忽略
        //   valueStr += prefixStr + dataValueStr;
        //   return;
        // }

        dataValueStr = prefixStr + '(' + dataValueStr + ')' + (strI !== '' ? " + '".concat(_replaceStrs(strI), "'") : '');
      } else {
        dataValueStr = '(' + dataValueStr + ')';
      }

      valueStr += dataValueStr;

      if (obj.isAll) {
        return false;
      }
    }, true);
  }

  return valueStr;
}

function _buildParams(node, fns, counter, useString, level, tagName) {
  //节点参数
  var params = node.params,
      paramsEx = node.paramsEx;
  var useStringF = fns.useString,
      hasPropsEx = paramsEx;

  var paramsStr = '',
      _paramsC,
      _tagProps;

  if (params || hasPropsEx) {
    _paramsC = counter._params++;
    _tagProps = '_params' + _paramsC;
    paramsStr = 'var ' + _tagProps + ' = ';

    if (params) {
      var paramKeys = Object.keys(params),
          len = paramKeys.length;
      paramsStr += '{\n';
      tools.each(paramKeys, function (k, i) {
        var valueStr = _buildProps(params[k], fns, useString, level);

        if (!useStringF && k === 'style') {
          //将style字符串转换为对象
          valueStr = "".concat(GLOBAL, ".sp(") + valueStr + ')';
        }

        var key = _replaceStrs(k);

        var onlyKey = params[k].onlyKey;

        if (!useStringF) {
          key = tranData.fixPropName(key);
        }

        paramsStr += "  '" + key + "': " + (!onlyKey ? valueStr : !useString ? 'true' : "'".concat(key, "'")) + (i < len - 1 ? ',\n' : '');
      }, false);
      paramsStr += '\n};\n';
    }

    if (hasPropsEx) {
      if (!params) {
        paramsStr += '{};\n';
      }

      if (paramsEx) {
        paramsStr += _buildContent(paramsEx.content, paramsEx, fns, counter, {
          _paramsE: true
        }, null, useString, tagName, _tagProps);
      }

      if (useString) {
        paramsStr += '\n' + _tagProps + " = ".concat(GLOBAL, ".ans(") + _tagProps + ');\n';
      }
    } else if (useString) {
      paramsStr += '\n' + _tagProps + " = ".concat(GLOBAL, ".ans({}, ") + _tagProps + ');\n';
    }
  }

  return [paramsStr, _paramsC];
}

function _buildNode(node, parent, fns, counter, retType, level, useStringLocal, isFirst, tagName, tagProps) {
  var fnStr = '';
  var useStringF = fns.useString;

  if (node.type === 'nj_plaintext') {
    //文本节点
    var valueStr = _buildProps(node.content[0], fns, useStringLocal, level);

    if (valueStr === '') {
      return fnStr;
    }

    var textStr = _buildRender(node, parent, 1, retType, {
      text: valueStr
    }, fns, level, useStringLocal, node.allowNewline, isFirst);

    if (useStringF) {
      fnStr += textStr;
    } else {
      //文本中的特殊字符需转义
      fnStr += (0, _escape.unescape)(textStr);
    }
  } else if (node.type === 'nj_ex') {
    //扩展标签节点
    var _exC = counter._ex++,
        _dataReferC = counter._dataRefer++,
        configE = _extension.extensionConfig[node.ex],
        exVarStr = '_ex' + _exC,
        globalExStr = "".concat(GLOBAL, ".x['").concat(node.ex, "']");

    var dataReferStr = '',
        fnHVarStr;

    if (configE && configE.onlyGlobal) {
      //只能从全局获取
      fnStr += '\nvar ' + exVarStr + ' = ' + globalExStr + ';\n';
    } else {
      //优先从context.data中获取
      fnHVarStr = '_fnH' + counter._fnH++;
      fnStr += '\nvar ' + exVarStr + ';\n';
      fnStr += 'var ' + fnHVarStr + " = ".concat(CONTEXT, ".d('").concat(node.ex, "', 0, true);\n");
      fnStr += 'if (' + fnHVarStr + ') {\n';
      fnStr += '  ' + exVarStr + ' = ' + fnHVarStr + '.value;\n';
      fnStr += '} else {\n';
      fnStr += '  ' + exVarStr + ' = ' + globalExStr + ';\n';
      fnStr += '}\n';
    }

    dataReferStr += 'var _dataRefer' + _dataReferC + ' = [\n';

    if (node.args) {
      //构建匿名参数
      tools.each(node.args, function (arg, i) {
        var valueStr = _buildProps(arg, fns, useStringLocal, level);

        dataReferStr += '  ' + valueStr + ',';
      }, true);
    } //hash参数


    var retP = _buildParams(node, fns, counter, useStringLocal, level, tagName),
        paramsStr = retP[0],
        _paramsC = retP[1];

    dataReferStr += _buildOptions(configE, useStringLocal, node, fns, level, paramsStr !== '' ? '_params' + _paramsC : null, tagName, tagProps);
    dataReferStr += '\n];\n'; //添加匿名参数

    if (paramsStr !== '') {
      dataReferStr += "".concat(GLOBAL, ".aa(_params") + _paramsC + ', _dataRefer' + _dataReferC + ');\n';
    }

    fnStr += paramsStr + dataReferStr;

    if (process.env.NODE_ENV !== 'production') {
      //如果扩展标签不存在则打印警告信息
      fnStr += "".concat(GLOBAL, ".tf(_ex").concat(_exC, ", '").concat(node.ex, "', 'ex');\n");
    } //渲染


    fnStr += _buildRender(node, parent, 2, retType, {
      _ex: _exC,
      _dataRefer: _dataReferC,
      fnH: fnHVarStr
    }, fns, level, useStringLocal, node.allowNewline, isFirst);
  } else {
    //元素节点
    //节点类型和typeRefer
    var _typeC = counter._type++,
        _tagName = '_type' + _typeC;

    var _type, _typeRefer;

    if (node.typeRefer) {
      var valueStrT = _buildProps(node.typeRefer, fns, level);

      _typeRefer = valueStrT;
      _type = node.typeRefer.props[0].prop.name;
    } else {
      _type = node.type;
    }

    var typeStr;

    if (!useStringF) {
      var _typeL = _type.toLowerCase(),
          subName = '';

      if (!_typeRefer && _typeL.indexOf('.') > -1) {
        var typeS = _type.split('.');

        _typeL = _typeL.split('.')[0];
        _type = typeS[0];
        subName = ", '".concat(typeS[1], "'");
      }

      typeStr = _typeRefer ? "".concat(GLOBAL, ".er(") + _typeRefer + ", '" + _typeL + "', ".concat(GLOBAL, ", '") + _type + "', ".concat(CONTEXT, ")") : "".concat(GLOBAL, ".e('") + _typeL + "', ".concat(GLOBAL, ", '") + _type + "', ".concat(CONTEXT) + subName + ')';
    } else {
      typeStr = _typeRefer ? "".concat(GLOBAL, ".en(") + _typeRefer + ", '" + _type + "')" : "'".concat(_type, "'");
    }

    fnStr += '\nvar _type' + _typeC + ' = ' + typeStr + ';\n'; //节点参数

    var _retP = _buildParams(node, fns, counter, useStringF, level, _tagName),
        _paramsStr = _retP[0],
        _paramsC2 = _retP[1];

    fnStr += _paramsStr;

    var _compParamC, _childrenC;

    if (!useStringF) {
      //组件参数
      _compParamC = counter._compParam++;
      fnStr += 'var _compParam' + _compParamC + ' = [_type' + _typeC + ', ' + (_paramsStr !== '' ? '_params' + _paramsC2 : 'null') + '];\n';
    } else {
      //子节点字符串
      _childrenC = counter._children++;
      fnStr += 'var _children' + _childrenC + " = '';\n";
    } //子节点


    fnStr += _buildContent(node.content, node, fns, counter, !useStringF ? {
      _compParam: '_compParam' + _compParamC
    } : {
      _children: '_children' + _childrenC
    }, useStringF && node.type === _core["default"].noWsTag ? null : level != null ? level + 1 : level, useStringLocal, _tagName); //渲染

    fnStr += _buildRender(node, parent, 3, retType, !useStringF ? {
      _compParam: _compParamC
    } : {
      _type: _typeC,
      _typeS: _type,
      _typeR: _typeRefer,
      _params: _paramsStr !== '' ? _paramsC2 : null,
      _children: _childrenC,
      _selfClose: node.selfCloseTag
    }, fns, level, useStringLocal, node.allowNewline, isFirst);
  }

  return fnStr;
}

function _buildContent(content, parent, fns, counter, retType, level, useStringLocal, tagName, tagProps) {
  var fnStr = '';

  if (!content) {
    return fnStr;
  }

  tools.each(content, function (node) {
    var useString = node.useString;
    fnStr += _buildNode(node, parent, fns, counter, retType, level, useString != null ? useString : useStringLocal, fns._firstNode && level == 0, tagName, tagProps);

    if (fns._firstNode) {
      //输出字符串时模板第一个节点前面不加换行符
      fns._firstNode = false;
    }
  }, true);
  return fnStr;
}

function _buildRender(node, parent, nodeType, retType, params, fns, level, useStringLocal, allowNewline, isFirst) {
  var retStr;
  var useStringF = fns.useString,
      useString = useStringLocal != null ? useStringLocal : useStringF,
      noLevel = level == null;

  switch (nodeType) {
    case 1:
      //文本节点
      retStr = (!useStringF || allowNewline || noLevel ? '' : isFirst ? parent.type !== 'nj_root' ? "".concat(GLOBAL, ".fl(").concat(CONTEXT, ") + ") : '' : "'\\n' + ") + _buildLevelSpace(level, fns, allowNewline) + _buildLevelSpaceRt(useStringF, isFirst || noLevel) + params.text;
      break;

    case 2:
      //扩展标签
      retStr = '_ex' + params._ex + '.apply(' + (params.fnH ? params.fnH + ' ? ' + params.fnH + ".source : ".concat(CONTEXT) : CONTEXT) + ', _dataRefer' + params._dataRefer + ')';
      break;

    case 3:
      //元素节点
      if (!useStringF) {
        retStr = "".concat(GLOBAL, ".H(_compParam") + params._compParam + ')';
      } else {
        if (allowNewline && allowNewline !== 'nlElem' || noLevel) {
          retStr = '';
        } else if (isFirst) {
          retStr = parent.type !== 'nj_root' ? "".concat(GLOBAL, ".fl(").concat(CONTEXT, ") + ") : '';
        } else {
          retStr = "'\\n' + ";
        }

        if (node.type !== _core["default"].textTag && node.type !== _core["default"].noWsTag) {
          var levelSpace = _buildLevelSpace(level, fns, allowNewline),
              content = node.content,
              hasTypeR = params._typeR,
              hasParams = params._params != null;

          retStr += levelSpace + _buildLevelSpaceRt(useStringF, isFirst || noLevel) + "'<" + (hasTypeR ? "' + _type" + params._type : params._typeS) + (hasParams ? (!hasTypeR ? "'" : '') + ' + _params' + params._params : '') + (hasTypeR || hasParams ? " + '" : '');

          if (!params._selfClose) {
            retStr += ">'";
            retStr += ' + _children' + params._children + ' + ';
            retStr += (!content || allowNewline || noLevel ? '' : "'\\n' + ") + (content ? levelSpace : '') + //如果子节点为空则不输出缩进空格和换行符
            _buildLevelSpaceRt(useStringF, noLevel) + "'</" + (hasTypeR ? "' + _type" + params._type + " + '" : params._typeS) + ">'";
          } else {
            retStr += " />'";
          }
        } else {
          retStr += '_children' + params._children;
        }
      }

      break;
  } //保存方式


  if (retType === '1') {
    return '\nreturn ' + retStr + ';';
  } else if (retType === '2') {
    if (!useString) {
      return '\nret.push(' + retStr + ');\n';
    } else {
      return '\nret += ' + retStr + ';\n';
    }
  } else if (retType._paramsE) {
    return '\n' + retStr + ';\n';
  } else {
    if (!useStringF) {
      return '\n' + retType._compParam + '.push(' + retStr + ');\n';
    } else {
      return '\n' + retType._children + ' += ' + retStr + ';\n';
    }
  }
}

function _buildLevelSpace(level, fns, allowNewline) {
  var ret = '';

  if (allowNewline && allowNewline !== 'nlElem') {
    return ret;
  }

  if (fns.useString && level != null && level > 0) {
    ret += "'";

    for (var i = 0; i < level; i++) {
      ret += '  ';
    }

    ret += "' + ";
  }

  return ret;
}

function _buildLevelSpaceRt(useString, noSpace) {
  if (useString && !noSpace) {
    return "".concat(GLOBAL, ".ls(").concat(CONTEXT, ") + ");
  }

  return '';
}

function buildRuntime(astContent, ast, useString) {
  var fns = {
    useString: useString,
    _no: 0,
    //扩展标签函数计数
    _firstNode: true
  };

  _buildFn(astContent, ast, fns, fns._no, null, 0);

  return fns;
}