"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.precompile = precompile;
exports.buildRenderH = exports.buildRender = exports.renderH = exports.render = exports.compileH = exports.compile = void 0;

var _core = _interopRequireDefault(require("../core"));

var tools = _interopRequireWildcard(require("../utils/tools"));

var _checkElem = require("../parser/checkElem");

var tranData = _interopRequireWildcard(require("../transforms/transformData"));

var _buildRuntime = require("./buildRuntime");

var _checkStringElem = require("../parser/checkStringElem");

var _createTmplRule = require("../utils/createTmplRule");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

//编译模板并返回转换函数
function _createCompile(outputH) {
  return function (tmpl, tmplKey, fileName, delimiters, tmplRule) {
    if (!tmpl) {
      return;
    }

    if (tools.isObject(tmplKey)) {
      var options = tmplKey;
      tmplKey = options.tmplKey;
      fileName = options.fileName;
      delimiters = options.delimiters;
      tmplRule = options.tmplRule;
    } //编译模板函数


    var tmplFns;

    if (tmplKey) {
      tmplFns = _core["default"].templates[tmplKey];
    }

    if (!tmplFns) {
      var isObj = tools.isObject(tmpl);
      var fns;

      if (isObj && tmpl.main) {
        //直接传入预编译模板
        fns = tmpl;
      } else {
        //编译AST
        var root;

        if (tmplKey) {
          root = _core["default"].asts[tmplKey];
        }

        if (!root) {
          //Can be directly introduced into the AST
          if (isObj && tmpl.type === 'nj_root') {
            root = tmpl;
          } else {
            if (!tmplRule) {
              tmplRule = delimiters ? (0, _createTmplRule.createTmplRule)(delimiters) : _core["default"].tmplRule;
            }

            root = _createAstRoot(); //Transform string template to preAst

            if (tools.isString(tmpl)) {
              tmpl = _checkStringElem.compileStringTmpl.call({
                tmplRule: tmplRule,
                outputH: outputH,
                onlyParse: true,
                fileName: fileName
              }, tmpl);
            } //分析传入参数并转换为节点树对象


            (0, _checkElem.checkElem)(tmpl._njTmpl, root, tmplRule);
          } //保存模板AST编译结果到全局集合中


          if (tmplKey) {
            _core["default"].asts[tmplKey] = root;
          }
        }

        fns = (0, _buildRuntime.buildRuntime)(root.content, root, !outputH);
      }

      tmplFns = tranData.template(fns, tmplKey); //保存模板函数编译结果到全局集合中

      if (tmplKey) {
        _core["default"].templates[tmplKey] = tmplFns;
      }
    }

    return tmplFns.main;
  };
}

var compile = _createCompile();

exports.compile = compile;

var compileH = _createCompile(true); //Create template root object


exports.compileH = compileH;

function _createAstRoot() {
  var root = tools.obj();
  root.type = 'nj_root';
  root.content = [];
  return root;
} //Precompile template


function precompile(tmpl, outputH) {
  var tmplRule = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _core["default"].tmplRule;
  var hasAst = arguments.length > 3 ? arguments[3] : undefined;

  var root = _createAstRoot();

  if (tmpl.quasis) {
    var _tmpl = tmpl,
        quasis = _tmpl.quasis,
        isExpression = _tmpl.isExpression,
        isCss = _tmpl.isCss;
    tmpl = _checkStringElem.compileStringTmpl.call({
      tmplRule: tmplRule,
      outputH: outputH,
      onlyParse: true,
      isExpression: isExpression,
      isCss: isCss
    }, quasis);
  } else if (tools.isString(tmpl)) {
    tmpl = _checkStringElem.compileStringTmpl.call({
      tmplRule: tmplRule,
      outputH: outputH,
      onlyParse: true
    }, tmpl);
  }

  (0, _checkElem.checkElem)(tmpl._njTmpl, root, tmplRule);
  var tmplFns = (0, _buildRuntime.buildRuntime)(root.content, root, !outputH);

  if (hasAst) {
    return {
      fns: tmplFns,
      ast: root
    };
  } else {
    return tmplFns;
  }
}

function _createRender(outputH) {
  return function (tmpl, options) {
    return (outputH ? compileH : compile)(tmpl, options ? {
      tmplKey: options.tmplKey ? options.tmplKey : tmpl._njTmplKey,
      fileName: options.fileName,
      delimiters: options.delimiters
    } : tmpl._njTmplKey).apply(null, tools.arraySlice(arguments, 1));
  };
}

var render = _createRender();

exports.render = render;

var renderH = _createRender(true);

exports.renderH = renderH;

function _buildRender(outputH) {
  return function (tmpl, params) {
    var tmplMainFn = (outputH ? compileH : compile)(tmpl, tmpl._njTmplKey);

    if (params) {
      var tmplFn = function tmplFn() {
        return tmplMainFn.apply(this, tools.arrayPush([params], arguments));
      };

      tools.defineProp(params, '_njParam', {
        value: true
      });
      tools.defineProps(tmplFn, {
        _njTmpl: {
          value: true
        }
      });
      return tmplFn;
    }

    return tmplMainFn;
  };
}

var buildRender = _buildRender();

exports.buildRender = buildRender;

var buildRenderH = _buildRender(true);

exports.buildRenderH = buildRenderH;
tools.assign(_core["default"], {
  compile: compile,
  compileH: compileH,
  precompile: precompile,
  render: render,
  renderH: renderH,
  buildRender: buildRender,
  buildRenderH: buildRenderH
});