"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createTaggedTmpl = createTaggedTmpl;
exports.createTaggedTmplH = createTaggedTmplH;
exports.template = template;
exports.expression = expression;
exports.css = css;
exports.taggedTmplH = exports.taggedTmpl = void 0;

var _core = _interopRequireDefault(require("../core"));

var _tools = require("../utils/tools");

var _checkStringElem = require("../parser/checkStringElem");

var _createTmplRule = require("../utils/createTmplRule");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function createTaggedTmpl() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var outputH = opts.outputH,
      delimiters = opts.delimiters,
      fileName = opts.fileName,
      isExpression = opts.isExpression,
      isCss = opts.isCss;
  var tmplRule = delimiters ? (0, _createTmplRule.createTmplRule)(delimiters) : _core["default"].tmplRule;
  return function (strs) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return _checkStringElem.compileStringTmpl.apply({
      tmplRule: tmplRule,
      outputH: outputH,
      fileName: fileName,
      isExpression: isExpression,
      isCss: isCss
    }, arguments);
  };
}

function createTaggedTmplH() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  opts.outputH = true;
  return createTaggedTmpl(opts);
}

var taggedTmpl = createTaggedTmpl();
exports.taggedTmpl = taggedTmpl;
var taggedTmplH = createTaggedTmplH();
exports.taggedTmplH = taggedTmplH;

function template(strs) {
  for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    args[_key2 - 1] = arguments[_key2];
  }

  return (_core["default"].outputH ? taggedTmplH : taggedTmpl).apply(null, arguments)();
}

var _taggedExpressionH = createTaggedTmplH({
  isExpression: true
});

function expression(strs) {
  for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    args[_key3 - 1] = arguments[_key3];
  }

  return _taggedExpressionH.apply(null, arguments)();
}

var _taggedCssH = createTaggedTmplH({
  isCss: true
});

function css(strs) {
  for (var _len4 = arguments.length, args = new Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    args[_key4 - 1] = arguments[_key4];
  }

  return _taggedCssH.apply(null, arguments)();
}

(0, _tools.assign)(_core["default"], {
  createTaggedTmpl: createTaggedTmpl,
  createTaggedTmplH: createTaggedTmplH,
  taggedTmpl: taggedTmpl,
  htmlString: taggedTmpl,
  taggedTmplH: taggedTmplH,
  html: taggedTmplH,
  template: template,
  expression: expression,
  css: css
});